@@ -1,11 +1,14 @@
 import math
 import copy
 from rpg_icon_generator.utils.vector import Vector
-from rpg_icon_generator.utils.color import hsv2rgb, colorDarken, colorRandomize, colorLerp, colorLighten, float_range, floatLerp
+from rpg_icon_generator.utils.color import Color
+from rpg_icon_generator.utils.misc import float_range, floatLerp
 from rpg_icon_generator.generator.__drawing import Drawing
 from rpg_icon_generator.utils.random import Random
 from rpg_icon_generator.utils.bound import Bound
-from rpg_icon_generator.utils.constants import RARITY_COLOR, RARITY_RANGE
+from rpg_icon_generator.utils.constants import RARITY_COLOR, RARITY_RANGE, RARITY_COLOR_SECONDARY
+from shapely.geometry.polygon import Polygon
+from shapely.geometry import Point
 
 class Generator(Drawing):
     def set_seed(self, s):
@@ -13,21 +16,22 @@ class Generator(Drawing):
     
     def set_drawing_bound(self, dimension, complexity):
         s = self._get_turtle_bound_offset_from_complexity(complexity)
+        self.center = self.dimension/2
         self.drawing_bound = Bound(0, 0, dimension, dimension)
         self.turtle_bound = Bound(s, s, dimension - 2*s, dimension - 2*s)
         self.dscale = self.turtle_bound.h / 32
 
     def _get_turtle_bound_offset_from_complexity(self, c):
         out =  math.ceil((0.4 + ((-0.4 / 100) * c)) * self.dimension)
-        if out < 5:
-            out = 5
+        if out < 7:
+            out = 7
         return out
     def _draw_crossguard_helper(self, params):
         # the color of the xguard
-        xguardColorLight = hsv2rgb(self.random.randomRange(
+        xguardColorLight = Color.hsv2rgb(self.random.randomRange(
             0, 360), self.random.randomFloatLow()*0.5, self.random.randomRangeFloat(0.7, 1))
         # the shadow color of the xguard
-        xguardColorDark = colorDarken(xguardColorLight, 0.6)
+        xguardColorDark = xguardColorLight.copy().colorDarken(0.6)
         # the amount of symmetry for the xguard
         xguardSymmetry = 0 if self.random.randomFloat() < 0.3 else 1
         # the thickness of the xguard
@@ -117,7 +121,7 @@ class Generator(Drawing):
                         coreDistance if dotProduct < 0 else bestPoint.widthT - coreDistance
                     darkAmt = distFromTop / \
                         (bestPoint.widthB + bestPoint.widthT)
-                    self.draw_pixel(x, y, colorLerp(
+                    self.draw_pixel(x, y, Color.colorLerp(
                         xguardColorLight, xguardColorDark, darkAmt))
         return {
             "colorLight": xguardColorLight,
@@ -135,14 +139,11 @@ class Generator(Drawing):
         # wavelength of the hilt texture ( in diagonal pixels)
         hiltWavelength = max(2, math.ceil(
             self.random.randomRange(3, 6) * self.dscale))
-        # amplitude of the hilt wave
-        hiltWaveAmplitude = math.ceil(
-            self.random.randomRange(1, 3) * self.dscale)
         # the color of the hilt
-        hiltColorLight = hsv2rgb(self.random.randomRange(
+        hiltColorLight = Color.hsv2rgb(self.random.randomRange(
             0, 360), self.random.randomFloat(), self.random.randomRangeFloat(0.7, 1))
         # the color of the hilt inner shadows
-        hiltColorDark = colorDarken(hiltColorLight, 1)
+        hiltColorDark = hiltColorLight.copy().colorDarken(1)
 
         # start location of the hilt(diagonal axis, diagonal pixels)
         hiltRadiusOdd = (hiltRadius % 2) != 0
@@ -150,7 +151,7 @@ class Generator(Drawing):
         for l in float_range(0, hiltParams["lengthDiag"], 0.5):
             al = hiltParams["startDiag"] + l
             gripWave = abs(math.cos(math.pi * 2 * l / hiltWavelength))
-            color = colorLerp(hiltColorDark, hiltColorLight, gripWave)
+            color = Color.colorLerp(hiltColorDark, hiltColorLight, gripWave)
 
             # determine draw parameters
             core = Vector(al + self.turtle_bound.x, self.drawing_bound.h - (al + self.turtle_bound.y))
@@ -169,8 +170,8 @@ class Generator(Drawing):
             # draw grip line
             for h in range(left, right+1):
                 darkenAmt = max(0, h + hiltRadius) / (hiltRadius*4)
-                self.draw_pixel(core.x + h, core.y + h,
-                                colorDarken(color, darkenAmt))
+                self.draw_pixel(core.x + h, core.y + h, color.copy().colorDarken(darkenAmt))
+        return hiltRadius
 
     def _draw_blade_helper(self, startDiag):
         # determines the angle of the taper of the blade tip(as a ratio of the blade length)
@@ -278,10 +279,9 @@ class Generator(Drawing):
             "s": self.random.randomFloatExtreme() * 0.6 if self.random.randomFloat() < 0.3 else 0,
             "v": self.random.randomRangeFloat(0.75, 1)
         }
-        colorBladeLinearTip = hsv2rgb(**colorBladeLinearTipHsv)
+        colorBladeLinearTip = Color.hsv2rgb(**colorBladeLinearTipHsv)
         # forward-axis color of the blade at the hilt
-        colorBladeLinearHilt = colorRandomize(
-            colorDarken(colorBladeLinearTip, 0.7), 16, self.random)
+        colorBladeLinearHilt = colorBladeLinearTip.copy().colorDarken(0.7).colorRandomize(16, self.random)
         # amount to lighten blade edge
         bladeEdgeLighten = 0.5
         # amount to darken blade far half
@@ -318,15 +318,15 @@ class Generator(Drawing):
                 useWidth = bestPoint.widthL if dotProduct < 0 else bestPoint.widthR
                 coreDistance = bestPoint.distanceTo(x, y)
                 if coreDistance <= useWidth or coreDistance <= minimumBladeWidth:
-                    color = colorLerp(
+                    color = Color.colorLerp(
                         colorBladeLinearHilt, colorBladeLinearTip, bestPoint.normalizedDist)
 
                     # do not change core
                     if bestPoint.x == x and bestPoint.y == y:
                         pass
                     else:
-                        edgeColor = colorLighten(color, bladeEdgeLighten)
-                        darkColor = colorDarken(color, bladeRightDarken)
+                        edgeColor = color.copy().colorLighten(bladeEdgeLighten)
+                        darkColor = color.copy().colorDarken(bladeRightDarken)
                         nonEdgeColor = darkColor if dotProduct > 0 else color
                         # lighten edge
                         if useWidth > bladeCoreEdgeExcludeWidth:
@@ -334,7 +334,7 @@ class Generator(Drawing):
                             edgeAmount = (
                                 coreDistance - edgeWidthMin) / bladeEdgeWidth
                             edgeAmount = 1 - (1-edgeAmount)*(1-edgeAmount)
-                            color = colorLerp(
+                            color = Color.colorLerp(
                                 nonEdgeColor, edgeColor, edgeAmount)
 
                     self.draw_pixel(x, y, color)
@@ -358,7 +358,7 @@ class Generator(Drawing):
                     highlightDist = highlightCenter.distanceTo(x, y)
                     darkAmt = 1-min(1, 0.8 * shadowDist / pommelRadius)
                     lightAmt = 1-min(1, highlightDist / pommelRadius)
-                    self.draw_pixel(x, y, colorLighten(colorLerp(pommelColorLight, pommelColorDark, darkAmt), lightAmt))
+                    self.draw_pixel(x, y, Color.colorLerp(pommelColorLight, pommelColorDark, darkAmt).colorLighten(lightAmt))
 
     def _draw_border(self):
         width = self.drawing_bound.w
@@ -368,52 +368,197 @@ class Generator(Drawing):
             for y in range(height):
                 pixel = self.get_pixel_data(x, y)
                 # if this pixel is empty or edge
-                if pixel[3] == 0 or x == 0 or x == width - 1 or y == 0 or y == height - 1:
+                if pixel.a == 0 or x == 0 or x == width - 1 or y == 0 or y == height - 1:
                     # and any orthogonal pixel isn't
                     nxPix = self.get_pixel_data(x-1, y)
                     nyPix = self.get_pixel_data(x, y-1)
                     pxPix = self.get_pixel_data(x+1, y)
                     pyPix = self.get_pixel_data(x, y+1)
-                    if x > 0 and nxPix[3] > 0 or x < width - 2 and pxPix[3] > 0 or y > 0 and nyPix[3] > 0 or y < height - 2 and pyPix[3] > 0:
+                    if x > 0 and nxPix.a > 0 or x < width - 2 and pxPix.a > 0 or y > 0 and nyPix.a > 0 or y < height - 2 and pyPix.a > 0:
                         border_pixels.append((x, y))
         
         for px, py in border_pixels:
-            self.draw_pixel(px, py, {"r": 0, "g": 0,  "b": 0, "a": 1})
+            self.draw_pixel(px, py, Color(0, 0, 0))
 
     def _get_rarity_from_complexity(self, c):
         for name, cplx in RARITY_RANGE.items():
             if cplx[0] <= c <= cplx[1]:
                 return name
 
-    def _get_border_size_from_complexity(self, complexity):
-        rSpan = 100
-        min_border = 1
-        border_span = (self.dimension*0.25) - min_border
-
-        # Convert the left range into a 0-1 range (float)
-        valueScaled = float(complexity) / float(rSpan)
-
-        # Convert the 0-1 range into a value in the right range.
-        return int(min_border + (valueScaled * border_span))
-
     def _draw_rarity_border(self, complexity):
-        lighten_factor = 0.4
+        border_size = 3
+        lighten_factor = 0.6
         rarity = self._get_rarity_from_complexity(complexity)
-        border_size = self._get_border_size_from_complexity(complexity)
-        master_color = RARITY_COLOR[rarity]
-        lighten_color = colorLighten(copy.copy(master_color), lighten_factor)
+        master_color = Color(**RARITY_COLOR[rarity])
+        secondary_color = Color(**RARITY_COLOR_SECONDARY[rarity])
+        lighten_color = master_color.copy()
         colors = [lighten_color]
         for i in range(1, border_size):
-            c = colorLighten(copy.copy(master_color),
-                             lighten_factor - (i * (lighten_factor/(border_size + 1))))
-            c["a"] -= i * (1/(border_size + 1))
+            c = master_color.copy().colorLighten(i * (lighten_factor/(border_size + 1)))
             colors.append(c)
         width = self.drawing_bound.w
         height = self.drawing_bound.h
         for x in range(width):
             for y in range(height):
-                pixel = self.get_pixel_data(x, y)
                 for n in range(border_size):
-                    if pixel[3] == 0 and (x == n or x == width - (1+n) or y == n or y == height - (1+n)):
+                    pixel = self.get_pixel_data(x, y)
+                    if pixel.a == 0 and (x == n or x == width - (1+n) or y == n or y == height - (1+n)):
                         self.draw_pixel(x, y, colors[n])
+        
+        self._draw_corner(colors, secondary_color, Vector(border_size, border_size), Vector(1, 1))
+        self._draw_corner(colors, secondary_color, Vector(self.drawing_bound.w - 1 - border_size, border_size), Vector(-1, 1))
+        self._draw_corner(colors, secondary_color, Vector(border_size, self.drawing_bound.h -1  - border_size), Vector(1, -1))
+        self._draw_corner(colors, secondary_color, Vector(self.drawing_bound.w - 1 - border_size, self.drawing_bound.h - 1 - border_size), Vector(-1, -1))
+
+    def _draw_corner(self, colors, secondary_color, pos, mult):
+        for x in range(3):
+            self.draw_pixel_safe(pos.x + (x*mult.x), pos.y, colors[-2])
+        for y in range(3):
+            self.draw_pixel_safe(pos.x, pos.y + (y*mult.y), colors[-2])
+        for x in range(4):
+            self.draw_pixel_safe(pos.x + (x*mult.x), pos.y + (3*mult.y), colors[-1])
+        for y in range(4):
+            self.draw_pixel_safe(pos.x + (3*mult.x), pos.y + (y*mult.y), colors[-1])
+
+        if mult.x == 1 and mult.y == 1:
+            offset = Vector(1, 2)
+        elif mult.x == -1 and mult.y == 1:
+            offset = Vector(-2, 2)
+        elif mult.x == 1 and mult.y == -1:
+            offset = Vector(1, -1)
+        elif mult.x == -1 and mult.y == -1:
+            offset = Vector(-2, -1)
+
+        self.draw_pixel_safe(pos.x + offset.x, pos.y + offset.y, secondary_color)
+        self.draw_pixel_safe(pos.x + offset.x, pos.y + offset.y - 1 , secondary_color)
+        self.draw_pixel_safe(pos.x + offset.x + 1 , pos.y + offset.y, secondary_color)
+        self.draw_pixel_safe(pos.x + offset.x + 1, pos.y + offset.y- 1, Color(255, 255, 255))
+
+        c = colors[-1].copy().colorLighten(0.2)
+        for x in range(4, int(self.center)):
+            self.draw_pixel_safe(pos.x + (x*mult.x), pos.y, c)
+        for y in range(4, int(self.center)):
+            self.draw_pixel_safe(pos.x, pos.y + (y*mult.y), c)
+
+        self.draw_pixel_safe(pos.x + (4*mult.x), pos.y + (1*mult.y), c)
+        self.draw_pixel_safe(pos.x + (1*mult.x), pos.y + (4*mult.y), c)
+
+
+    def _draw_axe_blade_helper(self, origine, offset, body_width=5, body_heigth=10, axe_width=15):
+        # the color of the axe
+        colorAxeLinearTipHsv = Color.hsv2rgb(
+            self.random.randomRangeFloat(0, 360),
+            self.random.randomFloatExtreme() * 0.6 if self.random.randomFloat() < 0.3 else 0,
+            self.random.randomRangeFloat(0.75, 1)
+        )
+        axeColorLight = colorAxeLinearTipHsv.copy().colorLighten(0.5)
+        # the shadow color of the axe
+        axeColorDark = colorAxeLinearTipHsv.copy().colorDarken(0.5)
+        # the amount of symmetry for the axe
+        axeSymmetry = 0 if self.random.randomFloat() < 0.3 else 1
+
+        # the amount of symmetry for the axe on second axis
+        axeSymmetry2 = 0 if self.random.randomFloat() < 0.2 else 1
+        # the thickness of the axe
+        axeThickness = self.random.randomRangeFloatHigh(1, 2.5)
+        # chance for the axe to acquire a curve (per pixel)
+        axeOmegaChance = 0.6
+        # max magnitude of axe omega add
+        axeOmegaAmount = math.pi/8
+        # maximum absolute axe omega
+        axeMaxOmega = ((axeThickness-1)**2 * math.pi/7)*0.2
+        # size of each step in sampling the axe curve
+        axeSampleStepSize = math.sqrt(2)
+        
+
+        angle_45 = math.cos(math.pi/4)
+
+        self.draw_red_pixel(origine.x, origine.y, 1)
+        # produce axe shape
+        currentPoint = Vector(self.center, self.center).addVector(
+            Vector(
+                angle_45 * offset,
+                angle_45 * offset)
+        )
+        currentPoint = [currentPoint, Vector(currentPoint)]
+        axeControlPoints = [[], []]
+        axeAngle = [math.pi * 3/4, -math.pi/4]
+        axeOmega = [0.08, -0.08]
+        for axeProgress in float_range(0, axe_width, axeSampleStepSize):
+            for side in range(2):
+                velocity = Vector(math.cos(axeAngle[side]), math.sin(axeAngle[side]))
+                newPoint = Vector(currentPoint[side])
+                if side == 1:
+                    symmetricPoint = Vector(
+                        currentPoint[0].y,
+                        currentPoint[0].x)
+                    newPoint.lerpTo(symmetricPoint, axeSymmetry)
+                newPoint.widthT = axeThickness/2
+                newPoint.widthB = axeThickness/2
+                newPoint.normal = Vector(velocity.y, -1 * velocity.x).multiplyScalar(side*2-1)
+                newPoint.dist = axeProgress
+                axeControlPoints[side].append(newPoint)
+            for side in range(2):
+                velocity = Vector(math.cos(axeAngle[side]), math.sin(axeAngle[side]))
+                if self.random.randomFloat() < axeOmegaChance:
+                    axeOmega[side] += self.random.randomRangeFloat(-axeOmegaAmount, axeOmegaAmount)
+                    axeOmega[side] = math.copysign(1, axeOmega[side]) * min(axeMaxOmega, abs(axeOmega[side]))
+                axeStep = Vector(velocity).multiplyScalar(axeSampleStepSize)
+                currentPoint[side].addVector(axeStep)
+                axeAngle[side] += axeOmega[side]
+
+        for side in range(2):
+            controlPoints = axeControlPoints[side]
+            for i in range(len(controlPoints)):
+                controlPoints[i].addVector(Vector(
+                    origine.x - self.center,
+                    origine.y - self.center))
+
+
+        # compute polygone
+        poly_node = []
+        mid_low = Vector(-angle_45 * body_heigth/2, angle_45 * body_heigth/2)
+        poly_node.append(origine.copy().addVector(mid_low.copy().addVector(Vector(-angle_45 * body_width/2, -angle_45 * body_width/2))))
+        poly_node.append(origine.copy().addVector(mid_low.copy().addVector(Vector(angle_45 * body_width/2, angle_45 * body_width/2))))
+
+        poly_node += axeControlPoints[0][::-1]
+        poly_node += axeControlPoints[1]
+
+
+        mid_high = Vector(angle_45 * body_heigth/2, -angle_45 * body_heigth/2)
+        poly_node.append(origine.copy().addVector(mid_high.copy().addVector(Vector(angle_45 * body_width/2, angle_45 * body_width/2))))
+        poly_node.append(origine.copy().addVector(mid_high.copy().addVector(Vector(-angle_45 * body_width/2, -angle_45 * body_width/2))))
+
+        if axeSymmetry2:
+            s1 = [Vector(self.drawing_bound.h - l.y, self.drawing_bound.w - l.x) for l in axeControlPoints[1][::-1]]
+            poly_node += s1
+            s2 = [Vector(self.drawing_bound.h - l.y, self.drawing_bound.w - l.x) for l in axeControlPoints[0]]
+            poly_node += s2
+            axeControlPoints.append(s1)
+            axeControlPoints.append(s2)
+
+        poly = Polygon([p.to_coord() for p in poly_node])
+
+        # for i, p in enumerate(poly_node):
+        #     c = Color.hsv2rgb(int((i/len(poly_node))*360), 1, 1)
+        #     self.draw_pixel(p.x, p.y, c)
+
+        for x in range(self.drawing_bound.w):
+            for y in range(self.drawing_bound.h):
+                # find the minimum distance to the axe core
+                # OPT: obviously inefficient
+                coreDistanceSq = 100000
+                bestPoint = None
+                for controlPoints in axeControlPoints:
+                    for i in range(len(controlPoints)):
+                        distanceSq = controlPoints[i].distanceToSq(x, y)
+                        if distanceSq < coreDistanceSq:
+                            coreDistanceSq = distanceSq
+                            bestPoint = controlPoints[i]
+                coreDistance = math.sqrt(coreDistanceSq)
+                pt = Point(x, y)
+                if poly.contains(pt):
+                    darkAmt = 0 if coreDistance<3 else 1 #/ (offset)
+                    self.draw_pixel(x, y, Color.colorLerp(axeColorLight, axeColorDark, darkAmt))
+        return (axeColorLight, axeColorDark)
 
